# Installation

Run `composer require dedi/sylius-contact-plugin`

Add the plugin and dependency classes to your `config/bundles.php` :

```php
<?php

return [
    //..
    Dedi\SyliusQuotePlugin\DediSyliusQuotePlugin::class => ['all' => true],
    Sylius\Bundle\GridBundle\SyliusGridBundle::class => ['all' => true],
    winzou\Bundle\StateMachineBundle\winzouStateMachineBundle::class => ['all' => true],
    // ..
];
```

Create `dedi_sylius_quote_plugin.yaml` file into `config/packages` folder to import the plugin's config

```yaml
# config/packages/dedi_sylius_contact_plugin.yaml
imports:
    - { resource: "@DediSyliusQuotePlugin/Resources/config/config.yaml" }
```

### Import the routes

```yaml
# config/routes/sylius_shop.yaml

...

dedi_sylius_quote_plugin:
    resource: "@DediSyliusQuotePlugin/Resources/config/shop_routing.yml"
```

```yaml
# config/routes/sylius_admin.yaml

...

dedi_sylius_quote_plugin:
    resource: "@DediSyliusQuotePlugin/Resources/config/admin_routing.yml"
```

### Install Assets
```bash
    bin/console assets:install
```

### Extending Sylius' entities

#### Order
```php
    use Dedi\SyliusQuotePlugin\Model\OrderInterface;
    use Dedi\SyliusQuotePlugin\Traits\QuoteAwareTrait;
    use Doctrine\ORM\Mapping as ORM;
    use Sylius\Component\Core\Model\Order as BaseOrder;

    /**
    * @ORM\Entity
    * @ORM\Table(name="sylius_order")
    */
    class Order extends BaseOrder implements OrderInterface
    {
        use QuoteAwareTrait;
    }
```

#### OrderItem
```php
    use Dedi\SyliusQuotePlugin\Model\OrderItemInterface;
    use Dedi\SyliusQuotePlugin\Traits\QuoteItemAwareTrait;
    use Doctrine\ORM\Mapping as ORM;
    use Sylius\Component\Core\Model\OrderItem as BaseOrderItem;

    /**
    * @ORM\Entity
    * @ORM\Table(name="sylius_order_item")
    */
    class OrderItem extends BaseOrderItem implements OrderItemInterface
    {
        use QuoteItemAwareTrait;
    }
```

### Extending Sylius' repositories

#### Product Variant Repository

In order to have the autocomplete work in the quote admin, extend Sylius' repository like this :

```php
    use Dedi\SyliusQuotePlugin\Traits\AutocompleteRepositoryTrait;
    use Sylius\Bundle\CoreBundle\Doctrine\ORM\ProductVariantRepository as BaseProductVariantRepository;
    
    class ProductVariantRepository extends BaseProductVariantRepository
    {
        use AutocompleteRepositoryTrait;
    }
```

### Create and execute database migrations

```bash
    bin/console doctrine:migrations:diff
    bin/console doctrine:migrations:migrate
```
