<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Modifier;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Entity\QuoteItemInterface;
use Dedi\SyliusQuotePlugin\Processor\Quote\QuoteProcessorInterface;

class QuoteModifier implements QuoteModifierInterface
{
    private QuoteItemQuantityModifierInterface $quoteItemQuantityModifier;

    private QuoteProcessorInterface $quoteProcessor;

    public function __construct(
        QuoteItemQuantityModifierInterface $quoteItemQuantityModifier,
        QuoteProcessorInterface $quoteProcessor
    ) {
        $this->quoteItemQuantityModifier = $quoteItemQuantityModifier;
        $this->quoteProcessor = $quoteProcessor;
    }

    public function addToQuote(QuoteInterface $quote, QuoteItemInterface $quoteItem): void
    {
        $this->resolveQuoteItem($quote, $quoteItem);

        $this->quoteProcessor->process($quote);
    }

    public function removeFromQuote(QuoteInterface $quote, QuoteItemInterface $quoteItem): void
    {
        $quote->removeItem($quoteItem);

        $this->quoteProcessor->process($quote);
    }

    private function resolveQuoteItem(QuoteInterface $quote, QuoteItemInterface $item): void
    {
        /** @var QuoteItemInterface $existingItem */
        foreach ($quote->getItems() as $existingItem) {
            if ($item->equals($existingItem)) {
                $this->quoteItemQuantityModifier->modify(
                    $existingItem,
                    $existingItem->getQuantity() + $item->getQuantity()
                );

                return;
            }
        }

        $quote->addItem($item);
    }
}
