<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\Voter;

use Dedi\SyliusQuotePlugin\Entity\QuoteInterface;
use Dedi\SyliusQuotePlugin\Specification\IsQuoteAcceptedSpecification;
use Dedi\SyliusQuotePlugin\Specification\IsQuoteUsableSpecification;
use Sylius\Component\Core\Model\AdminUserInterface;
use Sylius\Component\Core\Model\ShopUserInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;
use Symfony\Component\Security\Core\Security;

class QuoteVoter extends Voter
{
    public const IS_QUOTE_USABLE = 'IS_QUOTE_USABLE';

    public const IS_QUOTE_ACCEPTED = 'IS_QUOTE_ACCEPTED';

    private Security $security;

    private IsQuoteUsableSpecification $isQuoteUsableSpecification;

    private IsQuoteAcceptedSpecification $isQuoteAcceptedSpecification;

    public function __construct(
        Security $security,
        IsQuoteUsableSpecification $isQuoteUsableSpecification,
        IsQuoteAcceptedSpecification $isQuoteAcceptedSpecification
    ) {
        $this->security = $security;
        $this->isQuoteUsableSpecification = $isQuoteUsableSpecification;
        $this->isQuoteAcceptedSpecification = $isQuoteAcceptedSpecification;
    }

    protected function supports(string $attribute, $subject): bool
    {
        return in_array($attribute, [self::IS_QUOTE_USABLE, self::IS_QUOTE_ACCEPTED]) && $subject instanceof QuoteInterface;
    }

    /**
     * @param QuoteInterface $subject
     */
    protected function voteOnAttribute(string $attribute, $subject, TokenInterface $token): bool
    {
        $user = $this->security->getUser();
        if ($user instanceof AdminUserInterface) {
            return true;
        }

        if (!$user instanceof ShopUserInterface || $user->getCustomer() !== $subject->getCustomer()) {
            return false;
        }

        switch ($attribute) {
            case self::IS_QUOTE_USABLE:
                return $this->isQuoteUsableSpecification->__invoke($subject);
            case self::IS_QUOTE_ACCEPTED:
                return $this->isQuoteAcceptedSpecification->__invoke($subject);
            default:
                return true;
        }
    }
}
